/*
**  ClanLib SDK
**  Copyright (c) 1997-2005 The ClanLib Team
**
**  This software is provided 'as-is', without any express or implied
**  warranty.  In no event will the authors be held liable for any damages
**  arising from the use of this software.
**
**  Permission is granted to anyone to use this software for any purpose,
**  including commercial applications, and to alter it and redistribute it
**  freely, subject to the following restrictions:
**
**  1. The origin of this software must not be misrepresented; you must not
**     claim that you wrote the original software. If you use this software
**     in a product, an acknowledgment in the product documentation would be
**     appreciated but is not required.
**  2. Altered source versions must be plainly marked as such, and must not be
**     misrepresented as being the original software.
**  3. This notice may not be removed or altered from any source distribution.
**
**  Note: Some of the libraries ClanLib may link to may have additional
**  requirements or restrictions.
**
**  File Author(s):
**
**    AlexeyKharkov
**    (if your name is missing here, please add it)
*/

#include "precomp.h"
#include "API/Display/font.h"
#include "API/Display/display.h"
#include "API/Display/keys.h"
#include "API/Core/System/system.h"
#include "API/GUI/inputbox_stylebase.h"

int const SOME_LEFT_PIXELS = 25;

// Constructor
CL_InputBox_StyleBase::CL_InputBox_StyleBase(CL_InputBox *inputbox, CL_ResourceManager *resources, std::string const & font_name, int border_size)
: 
	inputbox(inputbox),
	border_size(border_size),
	show_cursor(false),
	draw_from(0)
{
	font = new CL_Font(font_name, resources);
	if (font)
		font->set_color( CL_Color::black );

	cursor_blink_time = CL_System::get_time();

	slot_input_down = inputbox->sig_input_down().connect(
		this, &CL_InputBox_StyleBase::on_input_down);
	slot_input_up = inputbox->sig_input_up().connect(
		this, &CL_InputBox_StyleBase::on_input_up);
	slot_mouse_move = inputbox->sig_mouse_move().connect(
		this, &CL_InputBox_StyleBase::on_mouse_move);
}

// Destructor
CL_InputBox_StyleBase::~CL_InputBox_StyleBase()
{
	delete font;
}

// -------------------------------------------------------------------------------------
void CL_InputBox_StyleBase::on_input_down(const CL_InputEvent &key)
{
	// Start selecting with mouse?
	if(key.id == CL_MOUSE_LEFT)
	{
		inputbox->on_mouse_Ldown( get_mouse_position(key.mouse_pos.x, key.mouse_pos.y) );
		return;
	}

	int old_cur_pos = inputbox->get_cursor_position();
	inputbox->on_input_down(key);
	int new_cur_pos = inputbox->get_cursor_position();

	if (new_cur_pos != old_cur_pos)
	{
		// update 'draw_from'
		std::string const & text = inputbox->get_text();
		int width = inputbox->get_width();

		if (new_cur_pos == 0)
		{
			draw_from = 0;
		} else
		if (new_cur_pos == text.length())
		{
			draw_from = text.length() - 1;
			int drawing_text_width = 0;

			while (draw_from > 0)
			{
				drawing_text_width += get_chars_width( draw_from, draw_from );

				if (drawing_text_width > width - border_size * 2 - SOME_LEFT_PIXELS)
					break;

				draw_from --;
			}
		} else
		if (new_cur_pos > old_cur_pos)
		{
			int shift = get_chars_width( draw_from, old_cur_pos );
			for (int i=old_cur_pos; i<=new_cur_pos; ++i)
			{
				int w = get_chars_width( i, i );
				shift += w;
				while (draw_from <= new_cur_pos  &&  
					shift + SOME_LEFT_PIXELS > width - border_size * 2)
				{
					shift -= get_chars_width( draw_from, draw_from );
					draw_from ++;
				}
			}
		} else
		{ // new_cur_pos < old_cur_pos
			int shift = get_chars_width( draw_from, old_cur_pos );
			for (int i=old_cur_pos; i>=new_cur_pos; --i)
			{
				int w = get_chars_width( i, i );
				shift -= w;
				while (draw_from > 0  &&  
					shift < SOME_LEFT_PIXELS + border_size * 2)
				{
					shift += get_chars_width( draw_from, draw_from );
					draw_from --;
				}
			}
		}
	}
}

void CL_InputBox_StyleBase::on_input_up(const CL_InputEvent &key)
{
	// End mouse-selecting
	if(key.id == CL_MOUSE_LEFT)
	{
		inputbox->on_mouse_Lup( get_mouse_position(key.mouse_pos.x, key.mouse_pos.y) );
		return;
	}
	inputbox->on_input_up(key);
}

void CL_InputBox_StyleBase::on_mouse_move(const CL_InputEvent &key)
{
	inputbox->on_mouse_move( get_mouse_position(key.mouse_pos.x, key.mouse_pos.y) );
}


int CL_InputBox_StyleBase::get_mouse_position(int x, int y)
{
	int delta_x = x;
	unsigned int i;
	std::string const & text = inputbox->get_text();

	for (i = draw_from; i < text.length(); i++)
	{
		int w = font->get_width( (unsigned char)text[i] );;
		delta_x -= w;
		if (delta_x <= -w/2)
		{
			return i;// + last_offset;
		}
	}
	return i;
}

int CL_InputBox_StyleBase::get_chars_width( int from, int to ) const
{
	std::string const & text = inputbox->get_text();

	if ( from < 0 ) from = 0;
	if ( to >= (int)text.length() ) to = text.length() - 1;
	if ( to < from ) return 0;

	if (inputbox->in_password_mode())
		return (to - from + 1) * font->get_width('*');

	int res = 0;
	for (int i=from; i<=to; ++i)
		res += font->get_width( (unsigned char)text[i] );

	return res;
}

